/**
 * Calibrate Rigol DP800 series power supplies via Telnet.
 * 
 * There is no need to use LXI, USB, RS232, UltraSigma, MATLAB or LabView
 * to calibrate the instrument since all Rigol instruments equiped with 
 * ethernet have Telnet access via port 5555.
 * 
 * Please configure the power supply with either a static IP address or
 * DHCP. If using DHCP, you can see the assigned address in the IO menu.
 * Note that DHCP requires a router or other networking device to assign
 * addresses. If only a "dumb" switch is used to connect the PC, DMM and
 * PSU, you will need to assign a static IP address for each device.
 * 
 * NOTE 1: The current script is written for the DP832/A. Other DP800 
 * series PSUs are supported, butwill need the cal point String array values
 * for the specific model being calibrated.These values are shown in the 
 * manual calibration menu of each instrument.
 * 
 * NOTE 2: This script was written for a Fluke/Tektronix DMM4050 6.5
 * digit DMM connected via Telnet at port 3490 and can easily be adapted
 * to support other Telnet enabled DMMs.
 * 
 * The SCPI calibration commands and procedure used in this script were
 * reverse-engineered by TooOldForThis and posted on the eevblog forum.
 *
 * @author garrettm
 * @version March 16th 2020, 3rd public release
 */

import java.text.SimpleDateFormat;

import java.util.Date;
import java.util.Scanner;
import java.util.concurrent.TimeUnit;
import java.util.InputMismatchException;

import java.net.InetAddress;
import java.net.UnknownHostException;

import java.io.IOException;

public class TelnetCal {
  
  // accept keyboard input from user
  private static Scanner input = null;
  
  // Telnet clients for each device
  private static TelnetClient psu = null;
  private static TelnetClient dmm = null;
    
  public static void main(String args[]) throws InterruptedException {
    input = new Scanner(System.in);    
    
    String psu_model = "DP832/A"; // model number of power supply
    int psu_port = 5555;          // telnet port for PSU (5555 is used for all Rigol instruments)
    
    String dmm_model = "DMM4050"; // model number of digital multimeter
    int dmm_port = 3490;          // telnet port for DMM (Tek DMM4050's default port is 3490)
    
    
    System.out.println("\n---------------- Rigol DP800 Series SCPI Calibration ----------------\n");
    Thread.sleep(500);
    
    System.out.println("This script uses Telnet access at port " + psu_port + " to connect to the "
                        + psu_model + "\nand port " + dmm_port + " for the " + dmm_model + "\n");
    Thread.sleep(500);
    
    System.out.println("Assuming DHCP is used, a router will assign an IP address to each\n"+
                       "device connected to the network");
    System.out.println("For example: 172.16.0.1 (router / DHCP server), 172.16.0.2 (PC),"+
                       "\n172.16.0.3 (PSU) and 172.16.0.4 (DMM)");
    Thread.sleep(500);
    
    
    System.out.println("\nEnter the IP address or hostname of the "+psu_model);
    InetAddress psu_addr = getAddr(); // address (IP/hostname) of PSU
    
    System.out.println("Opening Telnet connection at port "+psu_port+"...");
    psu = new TelnetClient(psu_addr, psu_port);
    System.out.println("Querying the instrument...");
    psu.send("*IDN?");
    System.out.println(psu.read());
    Thread.sleep(500);
    
    
    System.out.println("\nEnter the IP address or hostname of the "+dmm_model);
    InetAddress dmm_addr = getAddr(); // address (IP/hostname) of DMM
       
    System.out.println("Opening Telnet connection at port "+dmm_port+"...");
    dmm = new TelnetClient(dmm_addr, dmm_port);
    System.out.println("Querying the instrument...");
    dmm.send("*IDN?");
    System.out.println(dmm.read());
    Thread.sleep(500);
    
    
    System.out.println("\nPlease check that the identified instruments are correct before"+
                       "\ncontinuing further"+
                       "\n\nWhen ready, enter y to begin calibration or n to exit");
    if(input.next().equals("y")) {
      System.out.println("\n------------------------ START OF CALIBRATON ------------------------");
      Thread.sleep(500);
      Date start = new Date(); // calibration start time
	  
	  dmm.send("*RST"); // set DMM back to default configuration
      
      calChan(1);
      calChan(2);
      calChan(3);
	  
	  psu.send(":system:local"); // return to local mode
	  
      Date end = new Date(); // calibration end time
      long cal_duration = TimeUnit.MILLISECONDS.toMinutes(end.getTime() - start.getTime());
      
      System.out.println("\n------------------------ END OF CALIBRATION -------------------------\n");
      System.out.println("Time to complete calibration: "+cal_duration+" minutes\n");
    }
    
    
    // terminate telnet connection with instruments
    psu.close();
    dmm.close();
    
    // close scanner object
    input.close();
  }
  
  public static void calChan(int chan) throws InterruptedException {
    // voltage calibration points for channels 1 & 2
    String dac_v12[] = {"0.2v", "0.5v", "1.2v", "2v", "3.2v", "4.1v", "5.2v", "6.9v",
                        "7.5v", "8.7v", "10.1v", "11.8v", "12.6v", "13.5v", "15v", "15.8v",
                        "16.5v", "17.3v", "18.5v", "19.1v", "19.9v", "20.2v", "20.8v", "21.8v",
                        "22.4v", "22.7v", "23.9v", "24.3v", "25.7v", "26.9v", "27.9v", "28.5v",
                        "28.9v", "29.8v", "30.2v", "32v"};
    String adc_v12[] = {"0v", "0.05v", "0.1v", "0.5v", "1v", "5v", "10v", "12.8v", "20v", "30v", "32v"};
    
    // current calibration points for channels 1 & 2
    String dac_i12[] = {"0.1A", "0.25A", "0.5A", "0.8A", "1A", "1.25A", "1.5A", "1.75A", "1.9A", "2.15A",
                      "2.35A", "2.5A", "2.75A", "3A", "3.2A"};
    String adc_i12[] = {"0A", "0.01A", "0.1A", "1A", "2A", "3A", "3.2A"};
    
    // voltage calibration points for channel 3
    String dac_v3[] = {"0.1v", "0.2v", "0.4v", "0.85v", "1.2v", "1.8v", "2.55v", "3.1v", "3.4v", "4.1v",
                      "4.5v", "5v", "5.3v"};
    String adc_v3[] = {"0v", "0.005v", "0.01v", "0.02v", "0.05v", "0.1v", "0.5v", "1v", "3v", "5v", "5.3v"};
    
    // current calibration points for channel 3
    String dac_i3[] = {"0.1A", "0.5A", "1A", "1.25A", "1.5A", "1.75A", "2A", "2.25A", "2.5A", "2.75A",
                      "3A", "3.2A"};
    String adc_i3[] = {"0A", "0.1A", "1A", "2A", "3A", "3.2A"};
    
    
    String ch = (chan == 1) ? "ch1" : ((chan == 2) ? "ch2" : "ch3");
    
    // array aliases (makes string concantination easier)
    String dac_v[] = (chan==1 || chan==2) ? dac_v12 : dac_v3;
    String adc_v[] = (chan==1 || chan==2) ? adc_v12 : adc_v3;
    String dac_i[] = (chan==1 || chan==2) ? dac_i12 : dac_i3;
    String adc_i[] = (chan==1 || chan==2) ? adc_i12 : adc_i3;
    
    
    // SELECT CALIBRATION TYPE

    System.out.println("\n---------------------- SELECT CALIBRATION TYPE ----------------------\n");
    Thread.sleep(500);
    System.out.println("Note 1: If you're having trouble with voltage or current calibrations"+
                       "\nperforming a full voltage + current calibration will fix this");
    Thread.sleep(500);
    System.out.println("\nNote 2: Automated current calibration requires DMM with >= 3A range\n");
    Thread.sleep(500);
    System.out.println("Enter value of desired calibration type for "+ch+"\n"+
                       "\n0: skip "+ch+" calibration"+
                       "\n1: voltage + manual current calibration"+
                       "\n2: voltage + automated current calibration"+
                       "\n3: voltage calibration (automated entry)"+
                       "\n4: current calibration (manual entry)"+
                       "\n5: current calibration (automated entry)");
    
    int n;
    try {
      n = input.nextInt(); // obtain user selection
    } catch (InputMismatchException e) {n = 0;} // handle invalid entires
    
    if(n!=1 && n!=2 && n!=3 && n!=4 && n!=5) { // exit for any non valid cal value, not just 0
      System.out.println("\nCalibration for "+ch+" skipped");
      Thread.sleep(500);
      return;
    }
    
    
    // PREPARE INSTRUMENT FOR CALIBRATION
    
    psu.send("*rst");                         // clear errors before starting
    psu.send(":calibration:start 11111,"+ch); // begin SCPI calibration
    
    if (n==1 || n==2) { // full calibration routine
      psu.send(":calibration:clear "+ch+",all"); // clear all cal constants
	  System.out.println("\nCalibration constants for "+ch+" cleared");
    } else if (n == 3) { // voltage calibration routine              
      psu.send(":calibration:clear "+ch+",v"); // clear DAC-V and ADC-V cal constants
	  System.out.println("\nDAC-V and ADC-V calibration constants for "+ch+" cleared");
    } else { // current calibration routine
      psu.send(":calibration:clear "+ch+",c"); // clear DAC-I and ADC-I cal constants
	  System.out.println("\nDAC-I and ADC-I calibration constants for "+ch+" cleared");
    }
    
    psu.send("*rst");
    psu.send(":output "+ch+",on"); // enable output
    
    
    // VOLTAGE CALIBRATION ROUTINE
    
    if (n==1 || n==2 || n==3) {
      System.out.println("\n-------------------- VOLTAGE CALIBRATION ROUTINE --------------------\n");
      Thread.sleep(500);
      System.out.println("Please connect DMM voltage input to "+ch+" output of PSU");
      Thread.sleep(500);
      System.out.println("\nEnter 1 to begin automated voltage calibration");
      while(!input.next().equals("1")); // wait for user
    
      dmmRemVoltMeas(); // set DMM to remote voltage measurement mode
      
      System.out.println("\n"+ch+" DAC-V calibration");
      for (byte i=0; i<dac_v.length; i++) {
        // set cal point
        psu.send(":calibration:set "+ch+",v,"+Byte.toString(i)+","+dac_v[i]+",1");
        // wait for PSU to stabilize (value is in milliseconds and should be >= 2000)
        Thread.sleep(2000);
        //read value from dmm
        readDMM(); // dummy read to help settling due to autoranging
        String mes_val = readDMM();
        // send measured value
        psu.send(":calibration:meas "+ch+",v,"+Byte.toString(i)+","+mes_val+",1");
        System.out.printf("step %2d, cal point: %5s, meas val:%8.4fv%n",i,dac_v[i],Double.parseDouble(mes_val));
      }
      
      System.out.println("\n"+ch+" ADC-V calibration");
      for (byte i=0; i<adc_v.length; i++) {
        // set cal point
        psu.send(":calibration:set "+ch+",v,"+Byte.toString(i)+","+adc_v[i]+",0");
        // wait for PSU to stabilize (value is in milliseconds and should be >= 2000)
        Thread.sleep(2000);
        //read value from dmm
        readDMM(); // dummy read to help settling due to autoranging
        String mes_val = readDMM();
        // send measured value
        psu.send(":calibration:meas "+ch+",v,"+Byte.toString(i)+","+mes_val+",0");
        System.out.printf("step %2d, cal point: %5s, meas val:%8.4fv%n",i,adc_v[i],Double.parseDouble(mes_val));
      }
      
      if (n == 2) {
        dmmLoclCurrMeas(); // set DMM to local current measurement mode
      } else {
        dmmLoclVoltMeas(); // set DMM back to local voltage measurement mode
      }
    }
    
    
    // CURRENT CALIBRATION ROUTINES
    
    if(n==1 || n==4) { // manual current calibration
      System.out.println("\n-------------------- CURRENT CALIBRATION ROUTINE --------------------\n");
      Thread.sleep(500);
      
      // Note: the PSU output is set to 32v for ch1 & 2 after voltage calibration
      // to avoid sparking or damaging the current shunt terminals, we set the PSU to the
      // first cal step before connecting the output to be measured
      psu.send(":calibration:set "+ch+",c,0,"+dac_i[0]+",1");
      
      System.out.println("\nEnter 1 to begin manual current calibration");
      while(!input.next().equals("1")); // wait for user
      
      System.out.println("\n"+ch+" DAC-I calibration");
      for (byte i=0; i<dac_i.length; i++) {
        System.out.println("\nstep "+Byte.toString(i)+"/"+Integer.toString(dac_i.length-1)+", cal point: "+dac_i[i]);
        System.out.println("Enter 1 when ready to begin current measurement");
        while (!input.next().equals("1")); // wait before increasing current output
        // set cal point
        psu.send(":calibration:set "+ch+",c,"+Byte.toString(i)+","+dac_i[i]+",1");
        // manually enter measured value
        System.out.println("Enter measured current value");
        String mes_val = input.next();
        // send measured value
        psu.send(":calibration:meas "+ch+",c,"+Byte.toString(i)+","+mes_val+",1");
      }
    
      System.out.println("\n"+ch+" ADC-I calibration");
      for (byte i=0; i<adc_i.length; i++) {
        System.out.println("\nstep "+Byte.toString(i)+"/"+Integer.toString(adc_i.length-1)+", cal point: "+adc_i[i]);
        System.out.println("Enter y when ready to begin current measurement");
        while (!input.next().equals("1")); // wait before increasing current output
        // set cal point
        psu.send(":calibration:set "+ch+",c,"+Byte.toString(i)+","+adc_i[i]+",0");
        // manually enter measurment
        System.out.println("Enter measured current value");
        String mes_val = input.next();
        // send measured value
        psu.send(":calibration:meas "+ch+",c,"+Byte.toString(i)+","+mes_val+",0");
      }
      
    } else if (n==2 || n==5) { // automated current calibration
      System.out.println("\n-------------------- CURRENT CALIBRATION ROUTINE --------------------\n");
      Thread.sleep(500);
      
      // Note: the PSU output is set to 32v for ch1 & 2 after voltage calibration
      // to avoid sparking or damaging the DMM input terminals, we set the PSU to the
      // first cal step before connecting the output to the DMM for current measurements
      psu.send(":calibration:set "+ch+",c,0,"+dac_i[0]+",1");
      
      System.out.println("Connect "+ch+" output to DMM 10A current input");
      Thread.sleep(500);
      System.out.println("\nEnter 1 to begin automated current calibration");
      while(!input.next().equals("1")); // wait for user
      
      dmmRemCurrMeas(); // set dmm for remote current measurement 10A autorange (1A,3A,10A)
      
      System.out.println("\n"+ch+" DAC-I calibration");
      for (byte i=0; i<dac_i.length; i++) {
        // set cal point
        psu.send(":calibration:set "+ch+",c,"+Byte.toString(i)+","+dac_i[i]+",1");
        // wait for PSU to stabilize (value is in milliseconds and should be >= 2000)
        Thread.sleep(3000);
        //read value from dmm
        readDMM(); // dummy read to help settling due to autoranging
        String mes_val = readDMM();
        // send measured value
        psu.send(":calibration:meas "+ch+",c,"+Byte.toString(i)+","+mes_val+",1");
        System.out.printf("step %2d, cal point: %5s, meas val:%8.4fv%n",i,dac_i[i],Double.parseDouble(mes_val));
      }
    
      System.out.println("\n"+ch+" ADC-I calibration");
      for (byte i=0; i<adc_i.length; i++) {
        // set cal point
        psu.send(":calibration:set "+ch+",c,"+Byte.toString(i)+","+adc_i[i]+",0");
        // wait for PSU to stabilize (value is in milliseconds and should be >= 2000)
        Thread.sleep(3000);
        //read value from dmm
        readDMM(); // dummy read to help settling due to autoranging
        String mes_val = readDMM();
        // send measured value
        psu.send(":calibration:meas "+ch+",c,"+Byte.toString(i)+","+mes_val+",0");
        System.out.printf("step %2d, cal point: %5s, meas val:%8.4fv%n",i,adc_i[i],Double.parseDouble(mes_val));
      }
      
      if (ch.equals("ch3")) {
        dmmLoclCurrMeas(); // set DMM back to local current measurement
      } else {
        dmmLoclVoltMeas(); // set DMM back to local voltage measurement
      }
    }
    
    
    // SAVE CALIBRATION DATA
    
    psu.send(":output "+ch+",off"); // disable output
	
    // calibration date (month/day/year format)
    String cal_date = new SimpleDateFormat("MM/dd/yyyy").format(new Date());
    
    // save cal constants
    psu.send(":calibration:end "+cal_date+","+ch);
	
    System.out.println("\nCalibration constants saved for "+ch);
    Thread.sleep(500);
  }
  
  public static String readDMM() throws InterruptedException {
    dmm.send("*CLS");
    dmm.send("INIT");
    do { // wait for measurement to complete
      dmm.send("*OPC?");
    } while (!dmm.read().equals("1"));
    dmm.send("FETC?");
    return dmm.read().replaceAll("\\+",""); // remove "+" from values
  }
  
  public static void dmmRemVoltMeas() throws InterruptedException {
    // configure DMM for voltage measurements
    dmm.send("*CLS");
    dmm.send("SYST:REM");
    dmm.send("CONF:VOLT:DC");
    dmm.send("VOLT:DC:NPLC 100");
    dmm.send("TRIG:SOUR IMM");
    dmm.send("TRIG:DEL 0");
    dmm.send("TRIG:COUN 1");
    dmm.send("SAMP:COUN 1");
    Thread.sleep(500);
  }
  
  public static void dmmLoclVoltMeas() throws InterruptedException {
    // set DMM back to local mode
    dmm.send("*CLS");
    dmm.send("SYST:LOC");
    dmm.send("CONF:VOLT:DC");
    dmm.send("VOLT:DC:NPLC 10");
  }
  
  public static void dmmRemCurrMeas() throws InterruptedException {
    // configure DMM for current measurements
    dmm.send("*CLS");
    dmm.send("SYST:REM");
    dmm.send("CONF:CURR:DC 10");
    dmm.send("CURR:DC:RANG:AUTO ON");
    dmm.send("CURR:DC:NPLC 100");
    dmm.send("TRIG:SOUR IMM");
    dmm.send("TRIG:DEL 0");
    dmm.send("TRIG:COUN 1");
    dmm.send("SAMP:COUN 1");
    Thread.sleep(500);
  }
  
  public static void dmmLoclCurrMeas() throws InterruptedException {
    // set DMM back to local mode
    dmm.send("*CLS");
    dmm.send("SYST:LOC");
    dmm.send("CONF:CURR:DC 10");
    dmm.send("CURR:DC:RANG:AUTO ON");
    dmm.send("CURR:DC:NPLC 10");
  }
  
  public static InetAddress getAddr() throws InterruptedException {
    InetAddress addr = null;
    try {
      addr = InetAddress.getByName(input.next());
      if(addr.isReachable(2000)) {
        System.out.println("\n" + addr.getHostAddress() + " is online");
        Thread.sleep(500);
    } else {
        System.out.println("\n" + addr.getHostAddress() + " cannot be reached");
        Thread.sleep(2000);
        System.exit(1);
      }
    } catch (IOException e) {
      System.out.println("Invalid IP address or hostname");
      Thread.sleep(2000);
      System.exit(1);
    }
    return addr;
  }
}