/**
 * This script is intended to test readback of your DMM before starting the calibration script.
 * You will need to read your DMM's programming manual and edit the commands so that the script
 * can work correctly and then copy these new commands into the appropirate DMM methods in 
 * TelnetCal, recomplie and then run it and iterate as needed.
 *
 * @author garrettm
 * @version March 10th 2020, 2nd public release
 */

import java.text.SimpleDateFormat;

import java.util.Date;
import java.util.Scanner;
import java.util.concurrent.TimeUnit;
import java.util.InputMismatchException;

import java.net.InetAddress;
import java.net.UnknownHostException;

import java.io.IOException;

public class TelnetTest {
  
  // accept keyboard input from user
  private static Scanner input = null;
  
  // Telnet clients for each device
  private static TelnetClient psu = null;
  private static TelnetClient dmm = null;
    
  public static void main(String args[]) throws InterruptedException {
    input = new Scanner(System.in);    
    
    String psu_model = "DP832/A"; // model number of power supply
    int psu_port = 5555;          // telnet port for PSU (5555 is used for all Rigol instruments)
    
    String dmm_model = "DMM4050"; // model number of digital multimeter
    int dmm_port = 3490;          // telnet port for DMM (Tek DMM4050's default port is 3490)
    
    
    System.out.println("\n---------------- Rigol DP800 Series SCPI Readback Test ----------------\n");
    Thread.sleep(500);
    
    System.out.println("This script uses Telnet access at port " + psu_port + " to connect to the "
                        + psu_model + "\nand port " + dmm_port + " for the " + dmm_model + "\n");
    Thread.sleep(500);
    
    System.out.println("Assuming DHCP is used, a router will assign an IP address to each\n"+
                       "device connected to the network");
    System.out.println("For example: 172.16.0.1 (router / DHCP server), 172.16.0.2 (PC),"+
                       "\n172.16.0.3 (PSU) and 172.16.0.4 (DMM)");
    Thread.sleep(500);
    
    
    System.out.println("\nEnter the IP address or hostname of the "+psu_model);
    InetAddress psu_addr = getAddr(); // address (IP/hostname) of PSU
    
    System.out.println("Opening Telnet connection at port "+psu_port+"...");
    psu = new TelnetClient(psu_addr, psu_port);
    System.out.println("Querying the instrument...");
    psu.send("*IDN?");
    System.out.println(psu.read());
    Thread.sleep(500);
    
    
    System.out.println("\nEnter the IP address or hostname of the "+dmm_model);
    InetAddress dmm_addr = getAddr(); // address (IP/hostname) of DMM
       
    System.out.println("Opening Telnet connection at port "+dmm_port+"...");
    dmm = new TelnetClient(dmm_addr, dmm_port);
    System.out.println("Querying the instrument...");
    dmm.send("*IDN?");
    System.out.println(dmm.read());
    Thread.sleep(500);
    
    
    System.out.println("\nPlease check that the identified instruments are correct before"+
                       "\ncontinuing further"+
                       "\n\nWhen ready, enter y to begin test or n to exit");
    if(input.next().equals("y")) {
      System.out.println("\n---------------------------- START OF TEST ---------------------------");
      Thread.sleep(500);
	  
      Date start = new Date(); // start time
	  dmm.send("*RST"); 
	  
	  System.out.println("\nPlease disconnect all cables from the DMM");
	  
      System.out.println("\nEnter 1 to begin automated voltage readback tests");
      while(!input.next().equals("1")); // wait for user
	  
	  System.out.println("\nTesting remote voltage measurement");
	  Thread.sleep(1000);
	  
	  // configure DMM for remote voltage measurements
      dmm.send("*CLS");
      dmm.send("SYST:REM");
      dmm.send("CONF:VOLT:DC");
      dmm.send("VOLT:DC:NPLC 100");
      dmm.send("TRIG:SOUR IMM");
      dmm.send("TRIG:DEL 0");
      dmm.send("TRIG:COUN 1");
      dmm.send("SAMP:COUN 1");
	  
	  // readback 5 voltage measurements
	  dmm.send("*CLS");
      for (int i=0; i<5; i++) {
        dmm.send("INIT");
        do { // wait for measurement to complete
          dmm.send("*OPC?");
        } while (!dmm.read().equals("1"));
        dmm.send("FETC?");
		System.out.printf("meas val:%8.4fv%n",Double.parseDouble(dmm.read()));
      }
	  
      System.out.println("\nTesting local voltage measurement");
      Thread.sleep(1000);
	  
      // configure DMM for local voltage measurements
      dmm.send("*CLS");
      dmm.send("SYST:LOC");
      dmm.send("CONF:VOLT:DC");
      dmm.send("VOLT:DC:NPLC 10");
      System.out.println("\nPlease check that instrument is in DC volts, 10npcl, local mode");
      Thread.sleep(5000);
	  
      System.out.println("\nEnter 1 to begin automated current readback tests");
      while(!input.next().equals("1")); // wait for user
	  
      System.out.println("\nTesting remote 1A to 10A current measurement");
      Thread.sleep(1000);
	  
      // configure DMM for current measurements
      dmm.send("*CLS");
      dmm.send("SYST:REM");
      dmm.send("CONF:CURR:DC 10");
      dmm.send("CURR:DC:RANG:AUTO ON");
      dmm.send("CURR:DC:NPLC 100");
      dmm.send("TRIG:SOUR IMM");
      dmm.send("TRIG:DEL 0");
      dmm.send("TRIG:COUN 1");
      dmm.send("SAMP:COUN 1");

	  // readback 5 current measurements
	  dmm.send("*CLS");
      for (int i=0; i<5; i++) {
        dmm.send("INIT");
        do { // wait for measurement to complete
          dmm.send("*OPC?");
        } while (!dmm.read().equals("1"));
        dmm.send("FETC?");
		System.out.printf("meas val:%8.4fA%n",Double.parseDouble(dmm.read()));
      }
	  
      System.out.println("\nTesting local current measurement");
      Thread.sleep(1000);
	  
	  // configure DMM for local current measurements
      dmm.send("*CLS");
      dmm.send("SYST:LOC");
      dmm.send("CONF:CURR:DC 10");
      dmm.send("CURR:DC:RANG:AUTO ON");
      dmm.send("CURR:DC:NPLC 10");
      System.out.println("\nPlease check that instrument is in DC current, 10npcl, local mode");
      Thread.sleep(5000);
	  
      System.out.println("\nEnter y to test reading output of PSU cal points or n to skip");
      if(input.next().equals("y")) {
        calChan(1);
        //calChan(2); // ch2 and ch3 are identical in operation and don't need to be tested
        //calChan(3);
      }
	  
      psu.send(":system:local"); // return to local mode
	  
      Date end = new Date(); // end time
      long cal_duration = TimeUnit.MILLISECONDS.toMinutes(end.getTime() - start.getTime());
      
      System.out.println("\n---------------------------- END OF TEST ----------------------------\n");
      System.out.println("Time to complete test: "+cal_duration+" minutes");
	  System.out.println("\nPlease power-cycle the instrument to resume normal operation");
    }
    
    
    // terminate telnet connection with instruments
    psu.close();
    dmm.close();
    
    // close scanner object
    input.close();
  }
  
  public static void calChan(int chan) throws InterruptedException {
    // voltage calibration points for channels 1 & 2
    String dac_v12[] = {"0.2v", "0.5v", "1.2v", "2v", "3.2v", "4.1v", "5.2v", "6.9v",
                        "7.5v", "8.7v", "10.1v", "11.8v", "12.6v", "13.5v", "15v", "15.8v",
                        "16.5v", "17.3v", "18.5v", "19.1v", "19.9v", "20.2v", "20.8v", "21.8v",
                        "22.4v", "22.7v", "23.9v", "24.3v", "25.7v", "26.9v", "27.9v", "28.5v",
                        "28.9v", "29.8v", "30.2v", "32v"};
    String adc_v12[] = {"0v", "0.05v", "0.1v", "0.5v", "1v", "5v", "10v", "12.8v", "20v", "30v", "32v"};
    
    // current calibration points for channels 1 & 2
    String dac_i12[] = {"0.1A", "0.25A", "0.5A", "0.8A", "1A", "1.25A", "1.5A", "1.75A", "1.9A", "2.15A",
                      "2.35A", "2.5A", "2.75A", "3A", "3.2A"};
    String adc_i12[] = {"0A", "0.01A", "0.1A", "1A", "2A", "3A", "3.2A"};
    
    // voltage calibration points for channel 3
    String dac_v3[] = {"0.1v", "0.2v", "0.4v", "0.85v", "1.2v", "1.8v", "2.55v", "3.1v", "3.4v", "4.1v",
                      "4.5v", "5v", "5.3v"};
    String adc_v3[] = {"0v", "0.005v", "0.01v", "0.02v", "0.05v", "0.1v", "0.5v", "1v", "3v", "5v", "5.3v"};
    
    // current calibration points for channel 3
    String dac_i3[] = {"0.1A", "0.5A", "1A", "1.25A", "1.5A", "1.75A", "2A", "2.25A", "2.5A", "2.75A",
                      "3A", "3.2A"};
    String adc_i3[] = {"0A", "0.1A", "1A", "2A", "3A", "3.2A"};
    
    
    String ch = (chan == 1) ? "ch1" : ((chan == 2) ? "ch2" : "ch3");
    
    // array aliases (makes string concantination easier)
    String dac_v[] = (chan==1 || chan==2) ? dac_v12 : dac_v3;
    String adc_v[] = (chan==1 || chan==2) ? adc_v12 : adc_v3;
    String dac_i[] = (chan==1 || chan==2) ? dac_i12 : dac_i3;
    String adc_i[] = (chan==1 || chan==2) ? adc_i12 : adc_i3;
    
    
    // SELECT CALIBRATION TYPE

    System.out.println("\n---------------------- SELECT READBACK TEST TYPE ----------------------\n");
    Thread.sleep(500);
    System.out.println("Note 1: If you're having trouble with voltage or current calibrations"+
                       "\nperforming a full voltage + current calibration will fix this");
    Thread.sleep(500);
    System.out.println("\nNote 2: Automated current calibration requires DMM with >= 3A range\n");
    Thread.sleep(500);
    System.out.println("Enter value of desired test type for "+ch+"\n"+
                       "\n0: skip "+ch+" test"+
                       "\n1: voltage + manual current test"+
                       "\n2: voltage + automated current test"+
                       "\n3: voltage test (automated entry)"+
                       "\n4: current test (manual entry)"+
                       "\n5: current test (automated entry)");
    
    int n;
    try {
      n = input.nextInt(); // obtain user selection
    } catch (InputMismatchException e) {n = 0;} // handle invalid entires
    
    if(n!=1 && n!=2 && n!=3 && n!=4 && n!=5) { // exit for any non valid cal value, not just 0
      System.out.println("\nTest for "+ch+" skipped");
      Thread.sleep(500);
      return;
    }
    
    
    // PREPARE INSTRUMENT FOR TEST
    
    psu.send("*rst");                         // clear errors before starting
    psu.send(":calibration:start 11111,"+ch); // begin SCPI calibration
    psu.send("*rst");
    psu.send(":output "+ch+",on"); // enable output
    
    
    // VOLTAGE CALIBRATION ROUTINE
    
    if (n==1 || n==2 || n==3) {
      System.out.println("\n-------------------- VOLTAGE READBACK ROUTINE --------------------\n");
      Thread.sleep(500);
      System.out.println("Please connect DMM voltage input to "+ch+" output of PSU");
      Thread.sleep(500);
      System.out.println("\nEnter 1 to begin automated voltage readback");
      while(!input.next().equals("1")); // wait for user
    
      dmmRemVoltMeas(); // set DMM to remote measurement mode
      
      System.out.println("\n"+ch+" DAC-V readback");
      for (byte i=0; i<dac_v.length; i++) {
        // set cal point
        psu.send(":calibration:set "+ch+",v,"+Byte.toString(i)+","+dac_v[i]+",1");
        // wait for PSU to stabilize (value is in milliseconds and should be >= 2000)
        Thread.sleep(1000);
        //read value from dmm
		readDMM(); // dummy read
        String mes_val = readDMM();
        System.out.printf("step %2d, cal point: %5s, meas val:%8.4fv%n",i,dac_v[i],Double.parseDouble(mes_val));
      }
      
      System.out.println("\n"+ch+" ADC-V readback");
      for (byte i=0; i<adc_v.length; i++) {
        // set cal point
        psu.send(":calibration:set "+ch+",v,"+Byte.toString(i)+","+adc_v[i]+",0");
        // wait for PSU to stabilize (value is in milliseconds and should be >= 2000)
        Thread.sleep(1000);
        //read value from dmm
		readDMM(); // dummy read
        String mes_val = readDMM();
        System.out.printf("step %2d, cal point: %5s, meas val:%8.4fv%n",i,adc_v[i],Double.parseDouble(mes_val));
      }
      
      if (n == 2) {
        dmmLoclCurrMeas(); // set DMM to local current measurement mode
      } else {
        dmmLoclVoltMeas(); // set DMM back to local voltage measurement mode
      }
    }
    
    
    // CURRENT CALIBRATION ROUTINES
    
    if(n==1 || n==4) { // manual current calibration
      System.out.println("\n-------------------- CURRENT READBACK ROUTINE --------------------\n");
      Thread.sleep(500);
      System.out.println("DMM readback test is not avaliable for manual current entry");
    } else if (n==2 || n==5) { // automated current calibration
      System.out.println("\n-------------------- CURRENT READBACK ROUTINE --------------------\n");
      Thread.sleep(500);
      
      // Note: the PSU output is set to 32v for ch1 & 2 after voltage calibration
      // to avoid sparking or damaging the DMM input terminals, we set the PSU to the
      // first cal step before connecting the output to the DMM for current measurements
      psu.send(":calibration:set "+ch+",c,0,"+dac_i[0]+",1");
      
      System.out.println("Connect "+ch+" output to DMM 10A current input");
      Thread.sleep(500);
      System.out.println("\nEnter 1 to begin automated current readback");
      while(!input.next().equals("1")); // wait for user
      
      dmmRemCurrMeas(); // set dmm for remote current measurement 10A range (1A,3A,10A)
      
      System.out.println("\n"+ch+" DAC-I readback");
      for (byte i=0; i<dac_i.length; i++) {
        // set cal point
        psu.send(":calibration:set "+ch+",c,"+Byte.toString(i)+","+dac_i[i]+",1");
        // wait for PSU to stabilize (value is in milliseconds and should be >= 2000)
        Thread.sleep(1000);
        //read value from dmm
		readDMM(); // dummy read
        String mes_val = readDMM();
        System.out.printf("step %2d, cal point: %5s, meas val:%8.4fv%n",i,dac_i[i],Double.parseDouble(mes_val));
      }
    
      System.out.println("\n"+ch+" ADC-I readback");
      for (byte i=0; i<adc_i.length; i++) {
        // set cal point
        psu.send(":calibration:set "+ch+",c,"+Byte.toString(i)+","+adc_i[i]+",0");
        // wait for PSU to stabilize (value is in milliseconds and should be >= 2000)
        Thread.sleep(1000);
        //read value from dmm
		readDMM(); // dummy read
        String mes_val = readDMM();
        System.out.printf("step %2d, cal point: %5s, meas val:%8.4fv%n",i,adc_i[i],Double.parseDouble(mes_val));
      }
      
      if (ch.equals("ch3")) {
        dmmLoclCurrMeas(); // set DMM back to local current measurement
      } else {
        dmmLoclVoltMeas(); // set DMM back to local voltage measurement
      }
    }
	
	psu.send(":output "+ch+",off"); // disable output
    
  }
  
  public static String readDMM() throws InterruptedException {
    dmm.send("*CLS");
    dmm.send("INIT");
    do { // wait for measurement to complete
      dmm.send("*OPC?");
    } while (!dmm.read().equals("1"));
    dmm.send("FETC?");
    return dmm.read().replaceAll("\\+",""); // remove "+" from values
  }
  
  public static void dmmRemVoltMeas() throws InterruptedException {
    // configure DMM for voltage measurements
    dmm.send("*CLS");
    dmm.send("SYST:REM");
    dmm.send("CONF:VOLT:DC");
    dmm.send("VOLT:DC:NPLC 100");
    dmm.send("TRIG:SOUR IMM");
    dmm.send("TRIG:DEL 0");
    dmm.send("TRIG:COUN 1");
    dmm.send("SAMP:COUN 1");
    Thread.sleep(500);
  }
  
  public static void dmmLoclVoltMeas() throws InterruptedException {
    // set DMM back to local mode
    dmm.send("*CLS");
    dmm.send("SYST:LOC");
    dmm.send("CONF:VOLT:DC");
    dmm.send("VOLT:DC:NPLC 10");
  }
  
  public static void dmmRemCurrMeas() throws InterruptedException {
    // configure DMM for current measurements
    dmm.send("*CLS");
    dmm.send("SYST:REM");
    dmm.send("CONF:CURR:DC 10");
    dmm.send("CURR:DC:RANG:AUTO ON");
    dmm.send("CURR:DC:NPLC 100");
    dmm.send("TRIG:SOUR IMM");
    dmm.send("TRIG:DEL 0");
    dmm.send("TRIG:COUN 1");
    dmm.send("SAMP:COUN 1");
    Thread.sleep(500);
  }
  
  public static void dmmLoclCurrMeas() throws InterruptedException {
    // set DMM back to local mode
    dmm.send("*CLS");
    dmm.send("SYST:LOC");
    dmm.send("CONF:CURR:DC 10");
    dmm.send("CURR:DC:RANG:AUTO ON");
    dmm.send("CURR:DC:NPLC 10");
  }
  
  public static InetAddress getAddr() throws InterruptedException {
    InetAddress addr = null;
    try {
      addr = InetAddress.getByName(input.next());
      if(addr.isReachable(2000)) {
        System.out.println("\n" + addr.getHostAddress() + " is online");
        Thread.sleep(500);
    } else {
        System.out.println("\n" + addr.getHostAddress() + " cannot be reached");
        Thread.sleep(2000);
        System.exit(1);
      }
    } catch (IOException e) {
      System.out.println("Invalid IP address or hostname");
      Thread.sleep(2000);
      System.exit(1);
    }
    return addr;
  }
}